from __future__ import annotations

from typing import Literal, Optional, Union

from samtranslator.internal.schema_source.common import (
    BaseModel,
    PassThroughProp,
    ResourceAttributes,
    SamIntrinsicable,
    get_prop,
    passthrough_prop,
)

PROPERTIES_STEM = "sam-resource-layerversion"
CONTENT_URI_STEM = "sam-property-layerversion-layercontent"

contenturi = get_prop(CONTENT_URI_STEM)
properties = get_prop(PROPERTIES_STEM)


class ContentUri(BaseModel):
    Bucket: PassThroughProp = passthrough_prop(
        CONTENT_URI_STEM,
        "Bucket",
        ["AWS::Lambda::LayerVersion.Content", "S3Bucket"],
    )
    Key: PassThroughProp = passthrough_prop(
        CONTENT_URI_STEM,
        "Key",
        ["AWS::Lambda::LayerVersion.Content", "S3Key"],
    )
    Version: Optional[PassThroughProp] = passthrough_prop(
        CONTENT_URI_STEM,
        "Version",
        ["AWS::Lambda::LayerVersion.Content", "S3ObjectVersion"],
    )


class Properties(BaseModel):
    CompatibleArchitectures: Optional[PassThroughProp] = passthrough_prop(
        PROPERTIES_STEM,
        "CompatibleArchitectures",
        ["AWS::Lambda::LayerVersion", "Properties", "CompatibleArchitectures"],
    )
    CompatibleRuntimes: Optional[PassThroughProp] = passthrough_prop(
        PROPERTIES_STEM,
        "CompatibleRuntimes",
        ["AWS::Lambda::LayerVersion", "Properties", "CompatibleRuntimes"],
    )
    ContentUri: Union[str, ContentUri] = properties("ContentUri")
    Description: Optional[PassThroughProp] = passthrough_prop(
        PROPERTIES_STEM,
        "Description",
        ["AWS::Lambda::LayerVersion", "Properties", "Description"],
    )
    LayerName: Optional[PassThroughProp] = properties("LayerName")
    LicenseInfo: Optional[PassThroughProp] = passthrough_prop(
        PROPERTIES_STEM,
        "LicenseInfo",
        ["AWS::Lambda::LayerVersion", "Properties", "LicenseInfo"],
    )
    RetentionPolicy: Optional[SamIntrinsicable[str]] = properties("RetentionPolicy")


class Resource(ResourceAttributes):
    Type: Literal["AWS::Serverless::LayerVersion"]
    Properties: Properties



from __future__ import annotations

from typing import Any, Dict, Literal, Optional, Union

from samtranslator.internal.schema_source.common import (
    BaseModel,
    PassThroughProp,
    ResourceAttributes,
    SamIntrinsicable,
    get_prop,
    passthrough_prop,
)

PROPERTIES_STEM = "sam-resource-application"

location = get_prop("sam-property-application-applicationlocationobject")
properties = get_prop(PROPERTIES_STEM)


class Location(BaseModel):
    ApplicationId: SamIntrinsicable[str] = location("ApplicationId")
    SemanticVersion: SamIntrinsicable[str] = location("SemanticVersion")


class Properties(BaseModel):
    Location: Union[str, Location] = properties("Location")
    NotificationARNs: Optional[PassThroughProp] = passthrough_prop(
        PROPERTIES_STEM,
        "NotificationARNs",
        ["AWS::CloudFormation::Stack", "Properties", "NotificationARNs"],
    )
    Parameters: Optional[PassThroughProp] = passthrough_prop(
        PROPERTIES_STEM,
        "Parameters",
        ["AWS::CloudFormation::Stack", "Properties", "Parameters"],
    )
    Tags: Optional[Dict[str, Any]] = properties("Tags")
    TimeoutInMinutes: Optional[PassThroughProp] = passthrough_prop(
        PROPERTIES_STEM,
        "TimeoutInMinutes",
        ["AWS::CloudFormation::Stack", "Properties", "TimeoutInMinutes"],
    )


class Resource(ResourceAttributes):
    Type: Literal["AWS::Serverless::Application"]
    Properties: Properties



from samtranslator.model import GeneratedProperty, Resource
from samtranslator.model.intrinsics import ref


class NestedStack(Resource):
    resource_type = "AWS::CloudFormation::Stack"
    # TODO: support passthrough parameters for stacks (Conditions, etc)
    property_types = {
        "TemplateURL": GeneratedProperty(),
        "Parameters": GeneratedProperty(),
        "NotificationARNs": GeneratedProperty(),
        "Tags": GeneratedProperty(),
        "TimeoutInMinutes": GeneratedProperty(),
    }

    runtime_attrs = {"stack_id": lambda self: ref(self.logical_id)}



from abc import ABC, abstractmethod
from typing import Any, Dict


class Action(ABC):
    """
    Base class for Resolver function actions. Each Resolver function must subclass this,
    override the , and provide a execute() method
    """

    @abstractmethod
    def execute(self, template: Dict[str, Any]) -> Dict[str, Any]:
        pass


class ResolveDependsOn(Action):
    DependsOn = "DependsOn"

    def __init__(self, resolution_data: Dict[str, str]):
        """
        Initializes ResolveDependsOn. Where data necessary to resolve execute can be provided.

        :param resolution_data: Extra data necessary to resolve execute properly.
        """
        self.resolution_data = resolution_data

    def execute(self, template: Dict[str, Any]) -> Dict[str, Any]:
        """
        Resolve DependsOn when logical ids get changed when transforming (ex: AWS::Serverless::LayerVersion)

        :param input_dict: Chunk of the template that is attempting to be resolved
        :param resolution_data: Dictionary of the original and changed logical ids
        :return: Modified dictionary with values resolved
        """
        # Checks if input dict is resolvable
        if template is None or not self._can_handle_depends_on(input_dict=template):
            return template
        # Checks if DependsOn is valid
        if not (isinstance(template[self.DependsOn], (list, str))):
            return template
        # Check if DependsOn matches the original value of a changed_logical_id key
        for old_logical_id, changed_logical_id in self.resolution_data.items():
            # Done like this as there is no other way to know if this is a DependsOn vs some value named the
            # same as the old logical id. (ex LayerName is commonly the old_logical_id)
            if isinstance(template[self.DependsOn], list):
                for index, value in enumerate(template[self.DependsOn]):
                    if value == old_logical_id:
                        template[self.DependsOn][index] = changed_logical_id
            elif template[self.DependsOn] == old_logical_id:
                template[self.DependsOn] = changed_logical_id
        return template

    def _can_handle_depends_on(self, input_dict: Dict[str, Any]) -> bool:
        """
        Checks if the input dictionary is of length one and contains "DependsOn"

        :param input_dict: the Dictionary that is attempting to be resolved
        :return boolean value of validation attempt
        """
        return isinstance(input_dict, dict) and self.DependsOn in input_dict
